(* Copyright (c) 1992-1999, Microsoft Corporation, all rights reserved
**
** ras.h
** Remote Access external API
** Public header for external API clients
**
** WINVER values in this file:
**	WINVER < 0x400 = Windows NT 3.5, Windows NT 3.51
**	WINVER = 0x400 = Windows 95, Windows98, Windows NT4 (default) (WINVER400PLUS)
**	WINVER > 0x401 = Windows NT4 enhancements (WINNT4PLUS)
**	WINVER = 0x500 = Windows 2000 (WIN2000PLUS)
*)

//function prototypes got the prefix T

{$DEFINE WINVER400PLUS}

//includes all prior version
//Windows NT 3.51 is not supported ...
{$IFDEF WIN2000PLUS}{$DEFINE WINNT4PLUS}{$DEFINE WINVER400PLUS}{$ENDIF}
{$IFDEF WINNT4PLUS}{$DEFINE WINVER400PLUS}{$ENDIF}

//{$INCLUDE .\RasError.pas}
{$INCLUDE .\LMcons.pas}

const
  RAS_MaxDeviceType = 16;
  RAS_MaxPhoneNumber = 128;
  RAS_MaxIpAddress = 15;
  RAS_MaxIpxAddress = 21;

{$IFDEF WINVER400PLUS}
  RAS_MaxEntryName = 256;
  RAS_MaxDeviceName = 128;
  RAS_MaxCallbackNumber = RAS_MaxPhoneNumber;
{$ELSE}
  RAS_MaxEntryName = 20;
  RAS_MaxDeviceName = 32;
  RAS_MaxCallbackNumber = 48;
{$ENDIF}

  RAS_MaxAreaCode = 10;
  RAS_MaxPadType = 32;
  RAS_MaxX25Address = 200;
  RAS_MaxFacilities = 200;
  RAS_MaxUserData = 200;
  RAS_MaxReplyMessage = 1024;

type
  HRASCONN = DWORD;
  LPHRASCONN = ^HRASCONN;
  LPBOOL = ^BOOL;

// Identifies an active RAS connection. (See RasEnumConnections)

  RASCONNW = record
    dwSize: DWORD;
    hrasconn: HRASCONN;
    szEntryName: array[0..RAS_MaxEntryName] of WCHAR;
{$IFDEF WINVER400PLUS}
    szDeviceType: array[0..RAS_MaxDeviceType] of WCHAR;
    szDeviceName: array[0..RAS_MaxDeviceName] of WCHAR;
{$ENDIF}
{$IFDEF WINNT4PLUS}
    szPhonebook: array[0..MAX_PATH - 1] of WCHAR;
    dwSubEntry: DWORD;
{$ENDIF}
{$IFDEF WIN2000PLUS}
    guidEntry: GUID;
{$ENDIF}
  end;

  RASCONNA = record
    dwSize: DWORD;
    hrasconn: HRASCONN;
    szEntryName: array[0..RAS_MaxEntryName] of CHAR;
{$IFDEF WINVER400PLUS}
    szDeviceType: array[0..RAS_MaxDeviceType] of CHAR;
    szDeviceName: array[0..RAS_MaxDeviceName] of CHAR;
{$ENDIF}
{$IFDEF WINNT4PLUS}
    szPhonebook: array[0..MAX_PATH - 1] of CHAR;
    dwSubEntry: DWORD;
{$ENDIF}
{$IFDEF WIN2000PLUS}
    guidEntry: GUID;
{$ENDIF}
  end;

  LPRASCONNW = ^RASCONNW;
  LPRASCONNA = ^RASCONNA;
  LPRASCONN = ^RASCONN;
  RASCONN = RASCONNA;

const
// Enumerates intermediate states to a connection.(See RasDial)
  RASCS_PAUSED = $1000;
  RASCS_DONE = $2000;

  RASCS_OpenPort = 0;
  RASCS_PortOpened = 1;
  RASCS_ConnectDevice = 2;
  RASCS_DeviceConnected = 3;
  RASCS_AllDevicesConnected = 4;
  RASCS_Authenticate = 5;
  RASCS_AuthNotify = 6;
  RASCS_AuthRetry = 7;
  RASCS_AuthCallback = 8;
  RASCS_AuthChangePassword = 9;
  RASCS_AuthProject = 10;
  RASCS_AuthLinkSpeed = 11;
  RASCS_AuthAck = 12;
  RASCS_ReAuthenticate = 13;
  RASCS_Authenticated = 14;
  RASCS_PrepareForCallback = 15;
  RASCS_WaitForModemReset = 16;
  RASCS_WaitForCallback = 17;
  RASCS_Projected = 18;
{$IFDEF WINVER400PLUS}
  RASCS_StartAuthentication = 19;
  RASCS_CallbackComplete = 20;
  RASCS_LogonNetwork = 21;
  RASCS_SubEntryConnected = 22;
  RASCS_SubEntryDisconnected = 23;
{$ELSE}
  RASCS_SubEntryConnected = 19;
  RASCS_SubEntryDisconnected = 20;
{$ENDIF}
  RASCS_Interactive = RASCS_PAUSED;
  RASCS_RetryAuthentication = RASCS_PAUSED + 1;
  RASCS_CallbackSetByCaller = RASCS_PAUSED + 2;
  RASCS_PasswordExpired = RASCS_PAUSED + 3;
{$IFDEF WIN2000PLUS}
  RASCS_InvokeEapUI = RASCS_PAUSED + 4;
{$ENDIF}
  RASCS_Connected = RASCS_DONE;
  RASCS_Disconnected = RASCS_DONE + 1;

type
// Describes the status of a RAS connection.(See RasConnectionStatus)
  RASCONNSTATUSW = record
    dwSize: DWORD;
    rasconnstate: integer; //RASCONNSTATE
    dwError: DWORD;
    szDeviceType: array[0..RAS_MaxDeviceType] of WCHAR;
    szDeviceName: array[0..RAS_MaxDeviceName] of WCHAR;
{$IFDEF WINNT4PLUS}
    szPhoneNumber: array[0..RAS_MaxPhoneNumber] of WCHAR;
{$ENDIF}
  end;

  RASCONNSTATUSA = record
    dwSize: DWORD;
    rasconnstate: integer; //RASCONNSTATE
    dwError: DWORD;
    szDeviceType: array[0..RAS_MaxDeviceType] of CHAR;
    szDeviceName: array[0..RAS_MaxDeviceName] of CHAR;
{$IFDEF WINNT4PLUS}
    szPhoneNumber: array[0..RAS_MaxPhoneNumber] of CHAR;
{$ENDIF}
  end;

  LPRASCONNSTATUSW = ^RASCONNSTATUSW;
  LPRASCONNSTATUSA = ^RASCONNSTATUSA;
  LPRASCONNSTATUS = ^RASCONNSTATUS;
  RASCONNSTATUS = RASCONNSTATUSA;

// Describes connection establishment parameters.(See RasDial)

  RASDIALPARAMSW = record
    dwSize: DWORD;
    szEntryName: array[0..RAS_MaxEntryName] of WCHAR;
    szPhoneNumber: array[0..RAS_MaxPhoneNumber] of WCHAR;
    szCallbackNumber: array[0..RAS_MaxCallbackNumber] of WCHAR;
    szUserName: array[0..UNLEN] of WCHAR;
    szPassword: array[0..PWLEN] of WCHAR;
    szDomain: array[0..DNLEN] of WCHAR;
{$IFDEF WINNT4PLUS}
    dwSubEntry: DWORD;
    dwCallbackId: DWORD; //ULONG_PTR
{$ENDIF}
  end;

  RASDIALPARAMSA = record
    dwSize: DWORD;
    szEntryName: array[0..RAS_MaxEntryName] of CHAR;
    szPhoneNumber: array[0..RAS_MaxPhoneNumber] of CHAR;
    szCallbackNumber: array[0..RAS_MaxCallbackNumber] of CHAR;
    szUserName: array[0..UNLEN] of CHAR;
    szPassword: array[0..PWLEN] of CHAR;
    szDomain: array[0..DNLEN] of CHAR;
{$IFDEF WINNT4PLUS}
    dwSubEntry: DWORD;
    dwCallbackId: DWORD; //ULONG_PTR
{$ENDIF}
  end;

  LPRASDIALPARAMSW = ^RASDIALPARAMSW;
  LPRASDIALPARAMSA = ^RASDIALPARAMSA;
  LPRASDIALPARAMS = ^RASDIALPARAMS;
  RASDIALPARAMS = RASDIALPARAMSA;

{$IFDEF WIN2000PLUS}
  RASEAPINFO = record
    dwSizeofEapInfo: DWORD;
    pbEapInfo: PBYTE;
  end;
{$ENDIF}

// Describes extended connection establishment options.(See RasDial)
  RASDIALEXTENSIONS = record
    dwSize: DWORD;
    dwfOptions: DWORD;
    hwndParent: Cardinal; //HWND
    reserved: DWORD; //ULONG_PTR
{$IFDEF WIN2000PLUS}
    reserved1: DWORD; //ULONG_PTR
    RasEapInfo: RASEAPINFO;
{$ENDIF}
  end;

  LPRASDIALEXTENSIONS = ^RASDIALEXTENSIONS;

const
// 'dwfOptions' bit flags.
  RDEOPT_UsePrefixSuffix = $00000001;
  RDEOPT_PausedStates = $00000002;
  RDEOPT_IgnoreModemSpeaker = $00000004;
  RDEOPT_SetModemSpeaker = $00000008;
  RDEOPT_IgnoreSoftwareCompression = $00000010;
  RDEOPT_SetSoftwareCompression = $00000020;
  RDEOPT_DisableConnectedUI = $00000040;
  RDEOPT_DisableReconnectUI = $00000080;
  RDEOPT_DisableReconnect = $00000100;
  RDEOPT_NoUser = $00000200;
  RDEOPT_PauseOnScript = $00000400;
  RDEOPT_Router = $00000800;

{$IFDEF WIN2000PLUS}
  RDEOPT_CustomDial = $00001000;
{$ENDIF}

//
// This flag when set in the RASENTRYNAME structure
// indicates that the phonebook to which this entry
// belongs is a system phonebook.
//
  REN_User = $00000000;
  REN_AllUsers = $00000001;

type
// Describes an enumerated RAS phone book entry name.(See RasEntryEnum)
  RASENTRYNAMEW = record
    dwSize: DWORD;
    szEntryName: array[0..RAS_MaxEntryName] of WCHAR;
{$IFDEF WIN2000PLUS}
    // If this flag is REN_AllUsers then its a
    // system phonebook.
    dwFlags: DWORD;
    szPhonebookPath: array[0..MAX_PATH] of WCHAR;
{$ENDIF}
  end;

  RASENTRYNAMEA = record
    dwSize: DWORD;
    szEntryName: array[0..RAS_MaxEntryName] of CHAR;
{$IFDEF WIN2000PLUS}
    // If this flag is REN_AllUsers then its a
    // system phonebook.
    dwFlags: DWORD;
    szPhonebookPath: array[0..MAX_PATH] of CHAR;
{$ENDIF}
  end;

  LPRASENTRYNAMEW = ^RASENTRYNAMEW;
  LPRASENTRYNAMEA = ^RASENTRYNAMEA;
  LPRASENTRYNAME = ^RASENTRYNAME;
  RASENTRYNAME = RASENTRYNAMEA;

const
// Protocol code to projection data structure mapping.
  RASP_Amb = $10000;
  RASP_PppNbf = $803F;
  RASP_PppIpx = $802B;
  RASP_PppIp = $8021;
{$IFDEF WIN2000PLUS}
  RASP_PppCcp = $80FD;
{$ENDIF}
  RASP_PppLcp = $C021;
  RASP_Slip = $20000;

type
{
 Describes the result of a RAS AMB(Authentication Message Block)
 projection.This protocol is used with NT 3.1 and OS / 2 1.3 downlevel
 RAS servers.
}

  RASAMBW = record
    dwSize,
      dwError: DWORD;
    szNetBiosError: array[0..NETBIOS_NAME_LEN] of WCHAR;
    bLana: BYTE;
  end;

  RASAMBA = record
    dwSize,
      dwError: DWORD;
    szNetBiosError: array[0..NETBIOS_NAME_LEN] of CHAR;
    bLana: BYTE;
  end;

  LPRASAMBW = ^RASAMBW;
  LPRASAMBA = ^RASAMBA;
  LPRASAMB = ^RASAMB;
  RASAMB = RASAMBA;

// Describes the result of a PPP NBF(NetBEUI)projection.
  RASPPPNBFW = record
    dwSize,
      dwError,
      dwNetBiosError: DWORD;
    szNetBiosError: array[0..NETBIOS_NAME_LEN] of WCHAR;
    szWorkstationName: array[0..NETBIOS_NAME_LEN] of WCHAR;
    bLana: BYTE;
  end;

  RASPPPNBFA = record
    dwSize,
      dwError,
      dwNetBiosError: DWORD;
    szNetBiosError: array[0..NETBIOS_NAME_LEN] of CHAR;
    szWorkstationName: array[0..NETBIOS_NAME_LEN] of CHAR;
    bLana: BYTE;
  end;

  LPRASPPPNBFW = ^RASPPPNBFW;
  LPRASPPPNBFA = ^RASPPPNBFA;
  LPRASPPPNBF = ^RASPPPNBF;
  RASPPPNBF = RASPPPNBFA;

// Describes the results of a PPP IPX(Internetwork Packet Exchange) projection.
  RASPPPIPXW = record
    dwSize,
      dwError: DWORD;
    szIpxAddress: array[0..RAS_MaxIpxAddress] of WCHAR;
  end;

  RASPPPIPXA = record
    dwSize,
      dwError: DWORD;
    szIpxAddress: array[0..RAS_MaxIpxAddress] of CHAR;
  end;

  LPRASPPPIPXW = ^RASPPPIPXW;
  LPRASPPPIPXA = ^RASPPPIPXA;
  LPRASPPPIPX = ^RASPPPIPX;
  RASPPPIPX = RASPPPIPXA;

{$IFDEF WIN2000PLUS}
const
// Describes the results of a PPP IP(Internet)projection.
// RASPPPIP 'dwOptions' and 'dwServerOptions' flags.
  RASIPO_VJ = $00000001;
{$ENDIF}

type
  RASPPPIPW = record
    dwSize,
      dwError: DWORD;
    szIpAddress: array[0..RAS_MaxIpAddress] of WCHAR;
    (* This field was added between Windows NT 3.51 beta and Windows NT 3.51
    ** final, and between Windows 95 M8 beta and Windows 95 final.  If you do
    ** not require the server address and wish to retrieve PPP IP information
    ** from Windows NT 3.5 or early Windows NT 3.51 betas, or on early Windows
    ** 95 betas, define WINNT35COMPATIBLE.
    **
    ** The server IP address is not provided by all PPP implementations,
    ** though Windows NT server's do provide it.
    *)
    szServerIpAddress: array[0..RAS_MaxIpAddress] of WCHAR;
{$IFDEF WIN2000PLUS}
    dwOptions,
      dwServerOptions: DWORD;
{$ENDIF}
  end;

  RASPPPIPA = record
    dwSize,
      dwError: DWORD;
    szIpAddress: array[0..RAS_MaxIpAddress] of CHAR;
    (* This field was added between Windows NT 3.51 beta and Windows NT 3.51
    ** final, and between Windows 95 M8 beta and Windows 95 final.  If you do
    ** not require the server address and wish to retrieve PPP IP information
    ** from Windows NT 3.5 or early Windows NT 3.51 betas, or on early Windows
    ** 95 betas, define WINNT35COMPATIBLE.
    **
    ** The server IP address is not provided by all PPP implementations,
    ** though Windows NT server's do provide it.
    *)
    szServerIpAddress: array[0..RAS_MaxIpAddress] of CHAR;
{$IFDEF WIN2000PLUS}
    dwOptions,
      dwServerOptions: DWORD;
{$ENDIF}
  end;

  LPRASPPPIPW = ^RASPPPIPW;
  LPRASPPPIPA = ^RASPPPIPA;
  LPRASPPPIP = ^RASPPPIP;
  RASPPPIP = RASPPPIPA;

// Describes the results of a PPP LCP / multi - link negotiation.
{$IFDEF WIN2000PLUS}
const
// RASPPPLCP 'dwAuthenticatonProtocol' values.
  RASLCPAP_PAP = $C023;
  RASLCPAP_SPAP = $C027;
  RASLCPAP_CHAP = $C223;
  RASLCPAP_EAP = $C227;
// RASPPPLCP 'dwAuthenticatonData' values.
  RASLCPAD_CHAP_MD5 = $05;
  RASLCPAD_CHAP_MS = $80;
  RASLCPAD_CHAP_MSV2 = $81;
// RASPPPLCP 'dwOptions' and 'dwServerOptions' flags.
  RASLCPO_PFC = $00000001;
  RASLCPO_ACFC = $00000002;
  RASLCPO_SSHF = $00000004;
  RASLCPO_DES_56 = $00000008;
  RASLCPO_3_DES = $00000010;
{$ENDIF}

type
  RASPPPLCPW = record
    dwSize: DWORD;
    fBundled: BOOL;
{$IFDEF WIN2000PLUS}
    dwError,
      dwAuthenticationProtocol,
      dwAuthenticationData,
      dwEapTypeId,
      dwServerAuthenticationProtocol,
      dwServerAuthenticationData,
      dwServerEapTypeId: DWORD;
    fMultilink: BOOL;
    dwTerminateReason,
      dwServerTerminateReason: DWORD;
    szReplyMessage: array[0..RAS_MaxReplyMessage] of WCHAR;
    dwOptions,
      dwServerOptions: DWORD;
{$ENDIF}
  end;

  RASPPPLCPA = record
    dwSize: DWORD;
    fBundled: BOOL;
{$IFDEF WIN2000PLUS}
    dwError,
      dwAuthenticationProtocol,
      dwAuthenticationData,
      dwEapTypeId,
      dwServerAuthenticationProtocol,
      dwServerAuthenticationData,
      dwServerEapTypeId: DWORD;
    fMultilink: BOOL;
    dwTerminateReason,
      dwServerTerminateReason: DWORD;
    szReplyMessage: array[0..RAS_MaxReplyMessage] of CHAR;
    dwOptions,
      dwServerOptions: DWORD;
{$ENDIF}
  end;

  LPRASPPPLCPW = ^RASPPPLCPW;
  LPRASPPPLCPA = ^RASPPPLCPA;
  LPRASPPPLCP = ^RASPPPLCP;
  RASPPPLCP = RASPPPLCPA;

// Describes the results of a SLIP(Serial Line IP)projection.
  RASSLIPW = record
    dwSize,
      dwError: DWORD;
    szIpAddress: array[0..RAS_MaxIpAddress] of WCHAR;
  end;

  RASSLIPA = record
    dwSize,
      dwError: DWORD;
    szIpAddress: array[0..RAS_MaxIpAddress] of CHAR;
  end;

  LPRASSLIPW = ^RASSLIPW;
  LPRASSLIPA = ^RASSLIPA;
  LPRASSLIP = ^RASSLIP;
  RASSLIP = RASSLIPA;

{$IFDEF WIN2000PLUS}
const
// Describes the results of a PPP CCP(Compression Control Protocol) projection.
// RASPPPCCP 'dwCompressionAlgorithm' values.
  RASCCPCA_MPPC = $00000006;
  RASCCPCA_STAC = $00000005;
// RASPPPCCP 'dwOptions' values.
  RASCCPO_Compression = $00000001;
  RASCCPO_HistoryLess = $00000002;
  RASCCPO_Encryption56bit = $00000010;
  RASCCPO_Encryption40bit = $00000020;
  RASCCPO_Encryption128bit = $00000040;

type
  RASPPPCCP = record
    dwSize,
      dwError,
      dwCompressionAlgorithm,
      dwOptions,
      dwServerCompressionAlgorithm,
      dwServerOptions: DWORD;
  end;

  LPRASPPPCCP = ^RASPPPCCP;
{$ENDIF}

const
{
 if using RasDial message notifications, get the notification message code
 by passing this string to the RegisterWindowMessageA()API.
 WM_RASDIALEVENT is used only if a unique message cannot be registered.
}
  RASDIALEVENT = 'RasDialEvent';
  WM_RASDIALEVENT = $CCCD;

{
 Prototypes for caller's RasDial callback handler.  Arguments	are the
 message ID(currently always WM_RASDIALEVENT), the current RASCONNSTATE and
 the error that has occurred(or 0 if none).Extended arguments are the
 handle of the RAS connection and an extended error code.

 for RASDIALFUNC2, subsequent callback notifications for all
 subentries can be cancelled by returning FALSE.
}

type
  TRasDialFunc = procedure(
    unMsg, // type of event that has occurred
    rasconnstate, // connection state about to be entered
    dwError: DWORD // error that may have occurred
    ); stdcall;

  TRasDialFunc1 = procedure(
    hrasconn: HRASCONN; // handle to RAS connection
    unMsg, // type of event that has occurred
    rascs, // connection state about to be entered
    dwError, // error that may have occurred
    dwExtendedError: DWORD // extended error information for some errors
    ); stdcall;

  TRasDialFunc2 = function(
    dwCallbackId, // user-defined value specified in RasDial call
    dwSubEntry: DWORD; // subentry index in multilink connection
    hrasconn: HRASCONN; // handle to RAS connection
    unMsg, // type of event that has occurred
    rascs, // connection state about to be entered
    dwError, // error that may have occurred
    dwExtendedError: DWORD // extended error information for some errors
    ): DWORD; stdcall;

{
procedure RasDialFunc(
  unMsg, // type of event that has occurred
  rasconnstate, // connection state about to be entered
  dwError:DWORD // error that may have occurred
);stdcall;

procedure RasDialFunc1(
  hrasconn:HRASCONN; // handle to RAS connection
  unMsg, // type of event that has occurred
  rascs, // connection state about to be entered
  dwError, // error that may have occurred
  dwExtendedError:DWORD; // extended error information for some errors
);stdcall;

function RasDialFunc2(
  dwCallbackId, // user-defined value specified in RasDial call
  dwSubEntry:DWORD; // subentry index in multilink connection
  hrasconn:HRASCONN; // handle to RAS connection
  unMsg,	    // type of event that has occurred
  rascs,    // connection state about to be entered
  dwError,	   // error that may have occurred
  dwExtendedError:DWORD // extended error information for some errors
):DWORD;stdcall;
}

// Information describing a RAS - capable device.
  RASDEVINFOW = record
    dwSize: DWORD;
    szDeviceType: array[0..RAS_MaxDeviceType] of WCHAR;
    szDeviceName: array[0..RAS_MaxDeviceName] of WCHAR;
  end;

  RASDEVINFOA = record
    dwSize: DWORD;
    szDeviceType: array[0..RAS_MaxDeviceType] of CHAR;
    szDeviceName: array[0..RAS_MaxDeviceName] of CHAR;
  end;

  LPRASDEVINFOW = ^RASDEVINFOW;
  LPRASDEVINFOA = ^RASDEVINFOA;
  LPRASDEVINFO = ^RASDEVINFO;
  RASDEVINFO = RASDEVINFOA;

// RAS country information(currently retrieved from TAPI).
  RASCTRYINFO = record
    dwSize,
      dwCountryID,
      dwNextCountryID,
      dwCountryCode,
      dwCountryNameOffset: DWORD;
  end;

{
 There is currently no difference between
 RASCTRYINFOA and RASCTRYINFOW.This may
 change in the future.
}
  LPRASCTRYINFOW = ^RASCTRYINFOW;
  LPRASCTRYINFOA = ^RASCTRYINFOW;
  LPRASCTRYINFO = ^RASCTRYINFO;
  RASCTRYINFOW = RASCTRYINFO;
  RASCTRYINFOA = RASCTRYINFO;

// A RAS IP address.
  RASIPADDR = record
    a,
      b,
      c,
      d: BYTE;
  end;

const
{$IFDEF WIN2000PLUS}
  ET_40Bit = 1;
  ET_128Bit = 2;

  ET_None = 0; // No encryption
  ET_Require = 1; // Require Encryption
  ET_RequireMax = 2; // Require max encryption
  ET_Optional = 3; // Do encryption if possible. None Ok.
{$ENDIF}
  VS_Default = 0; // default (PPTP for now)
  VS_PptpOnly = 1; // Only	PPTP is attempted.
  VS_PptpFirst = 2; // PPTP is tried first.
  VS_L2tpOnly = 3; // Only	L2TP is attempted.
  VS_L2tpFirst = 4; // L2TP	is tried first.

type
// A RAS phone book entry.
  RASENTRYA = record
    dwSize,
      dwfOptions,
// Location/phone number.
    dwCountryID,
      dwCountryCode: DWORD;
    szAreaCode: array[0..RAS_MaxAreaCode] of CHAR;
    szLocalPhoneNumber: array[0..RAS_MaxPhoneNumber] of CHAR;
    dwAlternateOffset: DWORD;
// PPP/Ip
    ipaddr,
      ipaddrDns,
      ipaddrDnsAlt,
      ipaddrWins,
      ipaddrWinsAlt: RASIPADDR;
// Framing
    dwFrameSize,
      dwfNetProtocols,
      dwFramingProtocol: DWORD;
// Scripting
    szScript: array[0..MAX_PATH - 1] of CHAR;
// AutoDial
    szAutodialDll: array[0..MAX_PATH - 1] of CHAR;
    szAutodialFunc: array[0..MAX_PATH - 1] of CHAR;
// Device
    szDeviceType: array[0..RAS_MaxDeviceType] of CHAR;
    szDeviceName: array[0..RAS_MaxDeviceName] of CHAR;
// X.25
    szX25PadType: array[0..RAS_MaxPadType] of CHAR;
    szX25Address: array[0..RAS_MaxX25Address] of CHAR;
    szX25Facilities: array[0..RAS_MaxFacilities] of CHAR;
    szX25UserData: array[0..RAS_MaxUserData] of CHAR;
    dwChannels,
// Reserved
    dwReserved1,
      dwReserved2: DWORD;
{$IFDEF WINNT4PLUS}
// Multilink
    dwSubEntries,
      dwDialMode,
      dwDialExtraPercent,
      dwDialExtraSampleSeconds,
      dwHangUpExtraPercent,
      dwHangUpExtraSampleSeconds,
// Idle timeout
    dwIdleDisconnectSeconds: DWORD;
{$ENDIF}
{$IFDEF WIN2000PLUS}
// Entry Type
    dwType,
// Encryption type
    dwEncryptionType,
// CustomAuthKey to	be used for EAP
    dwCustomAuthKey: DWORD;
// Guid of the connection
    guidId: GUID;
// Custom Dial Dll
    szCustomDialDll: array[0..MAX_PATH - 1] of CHAR;
// DwVpnStrategy
    dwVpnStrategy: DWORD;
{$ENDIF}
  end;

  RASENTRYW = record
    dwSize,
      dwfOptions,
// Location/phone number.
    dwCountryID,
      dwCountryCode: DWORD;
    szAreaCode: array[0..RAS_MaxAreaCode] of WCHAR;
    szLocalPhoneNumber: array[0..RAS_MaxPhoneNumber] of WCHAR;
    dwAlternateOffset: DWORD;
// PPP/Ip
    ipaddr,
      ipaddrDns,
      ipaddrDnsAlt,
      ipaddrWins,
      ipaddrWinsAlt: RASIPADDR;
// Framing
    dwFrameSize,
      dwfNetProtocols,
      dwFramingProtocol: DWORD;
// Scripting
    szScript: array[0..MAX_PATH - 1] of WCHAR;
// AutoDial
    szAutodialDll: array[0..MAX_PATH - 1] of WCHAR;
    szAutodialFunc: array[0..MAX_PATH - 1] of WCHAR;
// Device
    szDeviceType: array[0..RAS_MaxDeviceType] of WCHAR;
    szDeviceName: array[0..RAS_MaxDeviceName] of WCHAR;
// X.25
    szX25PadType: array[0..RAS_MaxPadType] of WCHAR;
    szX25Address: array[0..RAS_MaxX25Address] of WCHAR;
    szX25Facilities: array[0..RAS_MaxFacilities] of WCHAR;
    szX25UserData: array[0..RAS_MaxUserData] of WCHAR;
    dwChannels,
// Reserved
    dwReserved1,
      dwReserved2: DWORD;
{$IFDEF WINNT4PLUS}
// Multilink
    dwSubEntries,
      dwDialMode,
      dwDialExtraPercent,
      dwDialExtraSampleSeconds,
      dwHangUpExtraPercent,
      dwHangUpExtraSampleSeconds,
// Idle timeout
    dwIdleDisconnectSeconds: DWORD;
{$ENDIF}
{$IFDEF WIN2000PLUS}
// Entry Type
    dwType,
// Encryption type
    dwEncryptionType,
// CustomAuthKey to	be used for EAP
    dwCustomAuthKey: DWORD;
// Guid of the connection
    guidId: GUID;
// Custom Dial Dll
    szCustomDialDll: array[0..MAX_PATH - 1] of WCHAR;
// DwVpnStrategy
    dwVpnStrategy: DWORD;
{$ENDIF}
  end;

  LPRASENTRYW = ^RASENTRYW;
  LPRASENTRYA = ^RASENTRYA;
  LPRASENTRY = ^RASENTRY;
  RASENTRY = RASENTRYA;

const
// RASENTRY 'dwfOptions' bit flags.
  RASEO_UseCountryAndAreaCodes = $00000001;
  RASEO_SpecificIpAddr = $00000002;
  RASEO_SpecificNameServers = $00000004;
  RASEO_IpHeaderCompression = $00000008;
  RASEO_RemoteDefaultGateway = $00000010;
  RASEO_DisableLcpExtensions = $00000020;
  RASEO_TerminalBeforeDial = $00000040;
  RASEO_TerminalAfterDial = $00000080;
  RASEO_ModemLights = $00000100;
  RASEO_SwCompression = $00000200;
  RASEO_RequireEncryptedPw = $00000400;
  RASEO_RequireMsEncryptedPw = $00000800;
  RASEO_RequireDataEncryption = $00001000;
  RASEO_NetworkLogon = $00002000;
  RASEO_UseLogonCredentials = $00004000;
  RASEO_PromoteAlternates = $00008000;
{$IFDEF WINNT4PLUS}
  RASEO_SecureLocalFiles = $00010000;
{$ENDIF}
{$IFDEF WIN2000PLUS}
  RASEO_RequireEAP = $00020000;
  RASEO_RequirePAP = $00040000;
  RASEO_RequireSPAP = $00080000;
  RASEO_Custom = $00100000;
  RASEO_PreviewPhoneNumber = $00200000;
  RASEO_SharedPhoneNumbers = $00800000;
  RASEO_PreviewUserPw = $01000000;
  RASEO_PreviewDomain = $02000000;
  RASEO_ShowDialingProgress = $04000000;
  RASEO_RequireCHAP = $08000000;
  RASEO_RequireMsCHAP = $10000000;
  RASEO_RequireMsCHAP2 = $20000000;
  RASEO_RequireW95MSCHAP = $40000000;
  RASEO_CustomScript = $80000000;
{$ENDIF}

// RASENTRY 'dwProtocols' bit flags.
  RASNP_NetBEUI = $00000001;
  RASNP_Ipx = $00000002;
  RASNP_Ip = $00000004;
// RASENTRY 'dwFramingProtocols' bit flags.
  RASFP_Ppp = $00000001;
  RASFP_Slip = $00000002;
  RASFP_Ras = $00000004;
// RASENTRY 'szDeviceType' default strings.
  RASDT_Modem = 'modem';
  RASDT_Isdn = 'isdn';
  RASDT_X25 = 'x25';
  RASDT_Vpn = 'vpn';
  RASDT_Pad = 'pad';
  RASDT_Generic = 'GENERIC';
  RASDT_Serial = 'SERIAL';
  RASDT_FrameRelay = 'FRAMERELAY';
  RASDT_Atm = 'ATM';
  RASDT_Sonet = 'SONET';
  RASDT_SW56 = 'SW56';
  RASDT_Irda = 'IRDA';
  RASDT_Parallel = 'PARALLEL';

// The entry type used to determine which UI properties
// are to be presented to user.  This generally corresponds
// to a Connections "add" wizard selection.
  RASET_Phone = 1; // Phone lines: modem, ISDN, X.25, etc
  RASET_Vpn = 2; // Virtual private network
  RASET_Direct = 3; // Direct connect: serial, parallel
  RASET_Internet = 4; // BaseCamp internet

type
{
  Old AutoDial

  DLLfunction prototype.
    This prototype is documented for backward - compatibility
    purposes only.It is superceded by the RASADFUNCA
    and RASADFUNCW definitions below. do not USE THIS
    PROTOTYPE in NEW CODE.SUPPORT for IT MAY BE REMOVED
    in FUTURE VERSIONS of RAS.
}

  TORASADFunc = function(
    hwndOwner: Cardinal; // handle of an owner window
    lpszEntry: Pchar; // pointer to a phone-book entry
    dwFlags: DWORD; // reserved; must be zero
    lpdwRetCode: LPDWORD // receives the results of a dialing operation
    ): BOOL; stdcall;

{$IFDEF WINNT4PLUS}
const
// Flags for RasConnectionNotification().
  RASCN_Connection = $00000001;
  RASCN_Disconnection = $00000002;
  RASCN_BandwidthAdded = $00000004;
  RASCN_BandwidthRemoved = $00000008;
// RASENTRY 'dwDialMode' values.
  RASEDM_DialAll = 1;
  RASEDM_DialAsNeeded = 2;
// RASENTRY 'dwIdleDisconnectSeconds' constants.
  RASIDS_Disabled = $FFFFFFFF;
  RASIDS_UseGlobalValue = 0;
// AutoDial

type
// DLLfunction parameter block.
  RASADPARAMS = record
    dwSize: DWORD;
    hwndOwner: Cardinal;
    dwFlags: DWORD;
    xDlg,
      yDlg: LongInt;
  end;

  LPRASADPARAMS = ^RASADPARAMS;

const
// AutoDial DLLfunction parameter block 'dwFlags.'
  RASADFLG_PositionDlg = $00000001;

type
// Prototype AutoDial DLLfunction.
  TRASADFuncA = function(
    lpszPhonebook, // pointer to full path and file name of phone-book file
    lpszEntry: PChar; // pointer to the entry name to validate
    lpAutoDialParams: PRASADPARAMS; // pointer to a RASADPARAMS structure
    lpdwRetCode: LPDWORD // receives results of dialing operation
    ): BOOL; stdcall;

  TRASADFuncW = function(
    lpszPhonebook, // pointer to full path and file name of phone-book file
    lpszEntry: PWideChar; // pointer to the entry name to validate
    lpAutoDialParams: PRASADPARAMS; // pointer to a RASADPARAMS structure
    lpdwRetCode: LPDWORD // receives results of dialing operation
    ): BOOL; stdcall;

  TRASADFUNC = TRASADFUNCA;

// A RAS phone book multilinked sub - entry.
  RASSUBENTRYA = record
    dwSize,
      dwfFlags: DWORD;
    // Device
    szDeviceType: array[0..RAS_MaxDeviceType] of CHAR;
    szDeviceName: array[0..RAS_MaxDeviceName] of CHAR;
    // Phone numbers
    szLocalPhoneNumber: array[0..RAS_MaxPhoneNumber] of CHAR;
    dwAlternateOffset: DWORD;
  end;

  RASSUBENTRYW = record
    dwSize,
      dwfFlags: DWORD;
    // Device
    szDeviceType: array[0..RAS_MaxDeviceType] of WCHAR;
    szDeviceName: array[0..RAS_MaxDeviceName] of WCHAR;
    // Phone numbers
    szLocalPhoneNumber: array[0..RAS_MaxPhoneNumber] of WCHAR;
    dwAlternateOffset: DWORD;
  end;

  LPRASSUBENTRYW = ^RASSUBENTRYW;
  LPRASSUBENTRYA = ^RASSUBENTRYA;
  LPRASSUBENTRY = ^RASSUBENTRY;
  RASSUBENTRY = RASSUBENTRYA;

// Ras {Get,Set}  Credentials structure.These calls supercede Ras {Get,Set}  EntryDialParams.
  RASCREDENTIALSA = record
    dwSize,
      dwMask: DWORD;
    szUserName: array[0..UNLEN] of CHAR;
    szPassword: array[0..PWLEN] of CHAR;
    szDomain: array[0..DNLEN] of CHAR;
  end;

  RASCREDENTIALSW = record
    dwSize,
      dwMask: DWORD;
    szUserName: array[0..UNLEN] of WCHAR;
    szPassword: array[0..PWLEN] of WCHAR;
    szDomain: array[0..DNLEN] of WCHAR;
  end;

  LPRASCREDENTIALSW = ^RASCREDENTIALSW;
  LPRASCREDENTIALSA = ^RASCREDENTIALSA;
  LPRASCREDENTIALS = ^RASCREDENTIALS;
  RASCREDENTIALS = RASCREDENTIALSA;

const
// RASCREDENTIALS 'dwMask' values.
  RASCM_UserName = $00000001;
  RASCM_Password = $00000002;
  RASCM_Domain = $00000004;

type
// AutoDial address properties.
  RASAUTODIALENTRYA = record
    dwSize,
      dwFlags,
      dwDialingLocation: DWORD;
    szEntry: array[0..RAS_MaxEntryName] of CHAR;
  end;

  RASAUTODIALENTRYW = record
    dwSize,
      dwFlags,
      dwDialingLocation: DWORD;
    szEntry: array[0..RAS_MaxEntryName] of WCHAR;
  end;

  LPRASAUTODIALENTRYW = ^RASAUTODIALENTRYW;
  LPRASAUTODIALENTRYA = ^RASAUTODIALENTRYA;
  LPRASAUTODIALENTRY = ^RASAUTODIALENTRY;
  RASAUTODIALENTRY = RASAUTODIALENTRYA;

const
// AutoDial control parameter values for Ras {Get,Set}	AutodialParam.
  RASADP_DisableConnectionQuery = 0;
  RASADP_LoginSessionDisable = 1;
  RASADP_SavedAddressesLimit = 2;
  RASADP_FailedConnectionTimeout = 3;
  RASADP_ConnectionQueryTimeout = 4;

{$ENDIF} // WINNT4PLUS
{$IFDEF WIN2000PLUS}
{
 RasGetEapUserIdentity bit flags.
 These have the same values as the RAS_EAP_FLAG_ flags in raseapif.h
}
  RASEAPF_NonInteractive = $00000002;
  RASEAPF_Logon = $00000004;
  RASEAPF_Preview = $00000008;

type
// RasGetEapUserIdentity structure.
  RASEAPUSERIDENTITYA = record
    szUserName: array[0..UNLEN] of CHAR;
    dwSizeofEapInfo: DWORD;
    pbEapInfo: array[0..0] of BYTE;
  end;

  RASEAPUSERIDENTITYW = record
    szUserName: array[0..UNLEN] of WCHAR;
    dwSizeofEapInfo: DWORD;
    pbEapInfo: array[0..0] of BYTE;
  end;

  LPRASEAPUSERIDENTITYW = ^RASEAPUSERIDENTITYW;
  LPRASEAPUSERIDENTITYA = ^RASEAPUSERIDENTITYA;
  RASEAPUSERIDENTITY = RASEAPUSERIDENTITYA;
{$ENDIF}

{
 External RAS
  APIfunction prototypes.
}

  TRasDialA = function(p1: pointer; p2: PChar; p3: LPRASDIALPARAMSA; p4: DWORD; p5: Pointer; p6: LPHRASCONN): DWORD; stdcall;
  TRasDialW = function(p1: pointer; p2: PWideChar; p3: LPRASDIALPARAMSW; p4: DWORD; p5: Pointer; p6: LPHRASCONN): DWORD; stdcall;
  TRasEnumConnectionsA = function(p1: LPRASCONNA; p2: LPDWORD; p3: LPDWORD): BOOL; stdcall;
  TRasEnumConnectionsW = function(p1: LPRASCONNW; p2: LPDWORD; p3: LPDWORD): BOOL; stdcall;
  TRasEnumEntriesA = function(p1: PChar; p2: PChar; p3: LPRASENTRYNAMEA; p4: LPDWORD; p5: LPDWORD): DWORD; stdcall;
  TRasEnumEntriesW = function(p1: PWideChar; p2: PWideChar; p3: LPRASENTRYNAMEW; p4: LPDWORD; p5: LPDWORD): DWORD; stdcall;
  TRasGetConnectStatusA = function(p1: HRASCONN; p2: LPRASCONNSTATUSA): DWORD; stdcall;
  TRasGetConnectStatusW = function(p1: HRASCONN; p2: LPRASCONNSTATUSW): DWORD; stdcall;
  TRasGetErrorStringA = function(p1: DWORD; p2: PChar; p3: DWORD): DWORD; stdcall;
  TRasGetErrorStringW = function(p1: DWORD; p2: PWideChar; p3: DWORD): DWORD; stdcall;
  TRasHangUpA = function(p1: HRASCONN): DWORD; stdcall;
  TRasHangUpW = function(p1: HRASCONN): DWORD; stdcall;
  TRasGetProjectionInfoA = function(p1: HRASCONN; p2: integer; p3: Pointer; p4: LPDWORD): DWORD; stdcall;
  TRasGetProjectionInfoW = function(p1: HRASCONN; p2: integer; p3: Pointer; p4: LPDWORD): DWORD; stdcall;
  TRasCreatePhonebookEntryA = function(HWND: Cardinal; p2: PChar): DWORD; stdcall;
  TRasCreatePhonebookEntryW = function(HWND: Cardinal; p2: PWideChar): DWORD; stdcall;
  TRasEditPhonebookEntryA = function(HWND: Cardinal; p2, p3: PChar): DWORD; stdcall;
  TRasEditPhonebookEntryW = function(HWND: Cardinal; p2, p3: PWideChar): DWORD; stdcall;
  TRasSetEntryDialParamsA = function(p1: PChar; p2: LPRASDIALPARAMSA; p3: BOOL): DWORD; stdcall;
  TRasSetEntryDialParamsW = function(p1: PWideChar; p2: LPRASDIALPARAMSW; p3: BOOL): DWORD; stdcall;
  TRasGetEntryDialParamsA = function(p1: PChar; p2: LPRASDIALPARAMSA; p3: LPBOOL): DWORD; stdcall;
  TRasGetEntryDialParamsW = function(p1: PWideChar; p2: LPRASDIALPARAMSW; p3: LPBOOL): DWORD; stdcall;
  TRasEnumDevicesA = function(p1: LPRASDEVINFOA; p2: LPDWORD; p3: LPDWORD): DWORD; stdcall;
  TRasEnumDevicesW = function(p1: LPRASDEVINFOW; p2: LPDWORD; p3: LPDWORD): DWORD; stdcall;
  TRasGetCountryInfoA = function(p1: LPRASCTRYINFOA; p2: LPDWORD): DWORD; stdcall;
  TRasGetCountryInfoW = function(p1: LPRASCTRYINFOW; p2: LPDWORD): DWORD; stdcall;
  TRasGetEntryPropertiesA = function(p1: PChar; p2: PChar; p3: LPRASENTRYA; p4: LPDWORD; p5: PBYTE; p6: LPDWORD): DWORD; stdcall;
  TRasGetEntryPropertiesW = function(p1: PWideChar; p2: PWideChar; p3: LPRASENTRYW; p4: LPDWORD; p5: PBYTE; p6: LPDWORD): DWORD; stdcall;
  TRasSetEntryPropertiesA = function(p1: PChar; p2: PChar; p3: LPRASENTRYA; p4: DWORD; p5: PBYTE; p6: DWORD): DWORD; stdcall;
  TRasSetEntryPropertiesW = function(p1: PWideChar; p2: PWideChar; p3: LPRASENTRYW; p4: DWORD; p5: PBYTE; p6: DWORD): DWORD; stdcall;
  TRasRenameEntryA = function(p1: PChar; p2: PChar; p3: PChar): DWORD; stdcall;
  TRasRenameEntryW = function(p1: PWideChar; p2: PWideChar; p3: PWideChar): DWORD; stdcall;
  TRasDeleteEntryA = function(p1: PChar; p2: PChar): DWORD; stdcall;
  TRasDeleteEntryW = function(p1: PWideChar; p2: PWideChar): DWORD; stdcall;
  TRasValidateEntryNameA = function(p1: PChar; p2: PChar): DWORD; stdcall;
  TRasValidateEntryNameW = function(p1: PWideChar; p2: PWideChar): DWORD; stdcall;
{$IFDEF WINNT4PLUS}
  TRasGetSubEntryHandleA = function(p1: HRASCONN; p2: DWORD; p3: LPHRASCONN): DWORD; stdcall;
  TRasGetSubEntryHandleW = function(p1: HRASCONN; p2: DWORD; p3: LPHRASCONN): DWORD; stdcall;
  TRasGetCredentialsA = function(p1: PChar; p2: PChar; p3: LPRASCREDENTIALSA): DWORD; stdcall;
  TRasGetCredentialsW = function(p1: PWideChar; p2: PWideChar; p3: LPRASCREDENTIALSW): DWORD; stdcall;
  TRasSetCredentialsA = function(p1: PChar; p2: PChar; p3: LPRASCREDENTIALSA; p4: BOOL): DWORD; stdcall;
  TRasSetCredentialsW = function(p1: PWideChar; p2: PWideChar; p3: LPRASCREDENTIALSW; p4: LPBOOL): DWORD; stdcall;
  TRasConnectionNotificationA = function(p1: HRASCONN; p2: Cardinal; p3: DWORD): DWORD; stdcall;
  TRasConnectionNotificationW = function(p1: HRASCONN; p2: Cardinal; p3: DWORD): DWORD; stdcall;
  TRasGetSubEntryPropertiesA = function(p1: PChar; p2: PChar; p3: DWORD; p4: RASSUBENTRYA; p5: LPDWORD; p6: PBYTE; p7: LPDWORD): DWORD; stdcall;
  TRasGetSubEntryPropertiesW = function(p1: PWideChar; p2: PWideChar; p3: DWORD; p4: RASSUBENTRYW; p5: LPDWORD; p6: PBYTE; p7: LPDWORD): DWORD; stdcall;
  TRasSetSubEntryPropertiesA = function(p1: PChar; p2: PChar; p3: DWORD; p4: LPRASSUBENTRYA; p5: DWORD; p6: PBYTE; p7: DWORD): DWORD; stdcall;
  TRasSetSubEntryPropertiesW = function(p1: PWideChar; p2: PWideChar; p3: DWORD; p4: LPRASSUBENTRYW; p5: DWORD; p6: PBYTE; p7: DWORD): DWORD; stdcall;
  TRasGetAutodialAddressA = function(p1: PChar; p2: LPDWORD; p3: LPRASAUTODIALENTRYA; p4: LPDWORD; p5: LPDWORD): DWORD; stdcall;
  TRasGetAutodialAddressW = function(p1: PWideChar; p2: LPDWORD; p3: LPRASAUTODIALENTRYW; p4: LPDWORD; p5: LPDWORD): DWORD; stdcall;
  TRasSetAutodialAddressA = function(p1: PChar; p2: DWORD; p3: LPRASAUTODIALENTRYA; p4: DWORD; p5: DWORD): DWORD; stdcall;
  TRasSetAutodialAddressW = function(p1: PWideChar; p2: DWORD; p3: LPRASAUTODIALENTRYW; p4: DWORD; p5: DWORD): DWORD; stdcall;
  TRasEnumAutodialAddressesA = function(p1: PChar; p2: LPDWORD; p3: LPDWORD): DWORD; stdcall;
  TRasEnumAutodialAddressesW = function(p1: PWideChar; p2: LPDWORD; p3: LPDWORD): DWORD; stdcall;
  TRasGetAutodialEnableA = function(p1: DWORD; p2: LPBOOL): DWORD; stdcall;
  TRasGetAutodialEnableW = function(p1: DWORD; p2: LPBOOL): DWORD; stdcall;
  TRasSetAutodialEnableA = function(p1: DWORD; p2: BOOL): DWORD; stdcall;
  TRasSetAutodialEnableW = function(p1: DWORD; p2: BOOL): DWORD; stdcall;
  TRasGetAutodialParamA = function(p1: DWORD; p2: Pointer; p3: LPDWORD): DWORD; stdcall;
  TRasGetAutodialParamW = function(p1: DWORD; p2: Pointer; p3: LPDWORD): DWORD; stdcall;
  TRasSetAutodialParamA = function(p1: DWORD; p2: Pointer; p3: DWORD): DWORD; stdcall;
  TRasSetAutodialParamW = function(p1: DWORD; p2: Pointer; p3: DWORD): DWORD; stdcall;
{$ENDIF}
{$IFDEF WIN2000PLUS}
  RAS_STATS = record
    dwSize,
      dwBytesXmited,
      dwBytesRcved,
      dwFramesXmited,
      dwFramesRcved,
      dwCrcErr,
      dwTimeoutErr,
      dwAlignmentErr,
      dwHardwareOverrunErr,
      dwFramingErr,
      dwBufferOverrunErr,
      dwCompressionRatioIn,
      dwCompressionRatioOut,
      dwBps,
      dwConnectDuration: DWORD;
  end;
  PRAS_STATS = ^RAS_STATS;

  TRasCustomHangUpFn = function(hRasConn: HRASCONN): DWORD; stdcall;

  TRasCustomDialFn = function(
    hInstDll: HINST;
    lpRasDialExtensions: PRASDIALEXTENSIONS;
    lpszPhonebook: PChar;
    lpRasDialParams: PRASDIALPARAMS;
    dwNotifierType: DWORD;
    lpvNotifier: Pointer;
    lphRasConn: PHRASCONN;
    dwFlags: DWORD
    ): DWORD; stdcall;

  TRasCustomDeleteEntryNotifyFn = function(
    lpszPhonebook,
    lpszEntry: PChar;
    dwFlags: DWORD
    ): DWORD; stdcall;

const
  RCD_SingleUser = 0;
  RCD_AllUsers = $00000001;
  RCD_Eap = $00000002;

type
  TRasInvokeEapUI = function(p1: HRASCONN; p2: DWORD; p3: LPRASDIALEXTENSIONS; p4: Cardinal): DWORD; stdcall;
  TRasGetLinkStatistics = function(hRasConn: HRASCONN; dwSubEntry: DWORD; lpStatistics: LPRAS_STATS): DWORD; stdcall;
  TRasGetConnectionStatistics = function(hRasConn: HRASCONN; lpStatistics: LPRAS_STATS): DWORD; stdcall;
  TRasClearLinkStatistics = function(hRasConn: HRASCONN; dwSubEntry: DWORD): DWORD; stdcall;
  TRasClearConnectionStatistics = function(hRasConn: HRASCONN): DWORD; stdcall;
  TRasGetEapUserDataA = function(hToken: Cardinal; pszPhonebook: PChar; pszEntry: PChar; pbEapData: PByte; pdwSizeofEapData: LPDWORD): DWORD; stdcall;
  TRasGetEapUserDataW = function(hToken: Cardinal; pszPhonebook: PWideChar; pszEntry: PWideChar; pbEapData: PByte; pdwSizeofEapData: LPDWORD): DWORD; stdcall;
  TRasSetEapUserDataA = function(hToken: Cardinal; pszPhonebook: PChar; pszEntry: PChar; pbEapData: PByte; dwSizeofEapData: DWORD): DWORD; stdcall;
  TRasSetEapUserDataW = function(hToken: Cardinal; pszPhonebook: PWideChar; pszEntry: PWideChar; pbEapData: PByte; dwSizeofEapData: DWORD): DWORD; stdcall;
  TRasGetCustomAuthDataA = function(pszPhonebook: PChar; pszEntry: PChar; pbCustomAuthData: PByte; pdwSizeofCustomAuthData: LPDWORD): DWORD; stdcall;
  TRasGetCustomAuthDataW = function(pszPhonebook: PWideChar; pszEntry: PWideChar; pbCustomAuthData: PByte; pdwSizeofCustomAuthData: LPDWORD): DWORD; stdcall;
  TRasSetCustomAuthDataA = function(pszPhonebook: PChar; pszEntry: PChar; pbCustomAuthData: PByte; dwSizeofCustomAuthData: DWORD): DWORD; stdcall;
  TRasSetCustomAuthDataW = function(pszPhonebook: PWideChar; pszEntry: PWideChar; pbCustomAuthData: PByte; dwSizeofCustomAuthData: DWORD): DWORD; stdcall;
  TRasGetEapUserIdentityW = function(pszPhonebook: PWideChar; pszEntry: PWideChar; dwFlags: DWORD; hwnd: Cardinal; ppRasEapUserIdentity: LPRASEAPUSERIDENTITYW): DWORD; stdcall;
  TRasGetEapUserIdentityA = function(pszPhonebook: PChar; pszEntry: PChar; dwFlags: DWORD; hwnd: Cardinal; ppRasEapUserIdentity: LPRASEAPUSERIDENTITYA): DWORD; stdcall;
  TRasFreeEapUserIdentityW = procedure(pRasEapUserIdentity: LPRASEAPUSERIDENTITYW); stdcall;
  TRasFreeEapUserIdentityA = procedure(pRasEapUserIdentity: LPRASEAPUSERIDENTITYA); stdcall;
{$ENDIF}

var
//Normal default to ANSI
  RasDial: TRasDialA = nil;
  RasEnumConnections: TRasEnumConnectionsA = nil;
  RasEnumEntries: TRasEnumEntriesA = nil;
  RasGetConnectStatus: TRasGetConnectStatusA = nil;
  RasGetErrorString: TRasGetErrorStringA = nil;
  RasHangUp: TRasHangUpA = nil;
  RasGetProjectionInfo: TRasGetProjectionInfoA = nil;
  RasCreatePhonebookEntry: TRasCreatePhonebookEntryA = nil;
  RasEditPhonebookEntry: TRasEditPhonebookEntryA = nil;
  RasSetEntryDialParams: TRasSetEntryDialParamsA = nil;
  RasGetEntryDialParams: TRasGetEntryDialParamsA = nil;
  RasEnumDevices: TRasEnumDevicesA = nil;
  RasGetCountryInfo: TRasGetCountryInfoA = nil;
  RasGetEntryProperties: TRasGetEntryPropertiesA = nil;
  RasSetEntryProperties: TRasSetEntryPropertiesA = nil;
  RasRenameEntry: TRasRenameEntryA = nil;
  RasDeleteEntry: TRasDeleteEntryA = nil;
  RasValidateEntryName: TRasValidateEntryNameA = nil;
{$IFDEF WINNT4PLUS}
  RasGetSubEntryHandle: TRasGetSubEntryHandleA = nil;
  RasConnectionNotification: TRasConnectionNotificationA = nil;
  RasGetSubEntryProperties: TRasGetSubEntryPropertiesA = nil;
  RasSetSubEntryProperties: TRasSetSubEntryPropertiesA = nil;
  RasGetCredentials: TRasGetCredentialsA = nil;
  RasSetCredentials: TRasSetCredentialsA = nil;
  RasGetAutodialAddress: TRasGetAutodialAddressA = nil;
  RasSetAutodialAddress: TRasSetAutodialAddressA = nil;
  RasEnumAutodialAddresses: TRasEnumAutodialAddressesA = nil;
  RasGetAutodialEnable: TRasGetAutodialEnableA = nil;
  RasSetAutodialEnable: TRasSetAutodialEnableA = nil;
  RasGetAutodialParam: TRasGetAutodialParamA = nil;
  RasSetAutodialParam: TRasSetAutodialParamA = nil;
{$ENDIF}
{$IFDEF WIN2000PLUS}
  RasGetEapUserData: TRasGetEapUserDataA = nil;
  RasSetEapUserData: TRasSetEapUserDataA = nil;
  RasGetCustomAuthData: TRasGetCustomAuthDataA = nil;
  RasSetCustomAuthData: TRasSetCustomAuthDataA = nil;
  RasGetEapUserIdentity: TRasGetEapUserIdentityA = nil;
  RasFreeEapUserIdentity: TRasFreeEapUserIdentityA = nil;
{$ENDIF}

//ANSI versions
  RasDialA: TRasDialA = nil;
  RasEnumConnectionsA: TRasEnumConnectionsA = nil;
  RasEnumEntriesA: TRasEnumEntriesA = nil;
  RasGetConnectStatusA: TRasGetConnectStatusA = nil;
  RasGetErrorStringA: TRasGetErrorStringA = nil;
  RasHangUpA: TRasHangUpA = nil;
  RasGetProjectionInfoA: TRasGetProjectionInfoA = nil;
  RasCreatePhonebookEntryA: TRasCreatePhonebookEntryA = nil;
  RasEditPhonebookEntryA: TRasEditPhonebookEntryA = nil;
  RasSetEntryDialParamsA: TRasSetEntryDialParamsA = nil;
  RasGetEntryDialParamsA: TRasGetEntryDialParamsA = nil;
  RasEnumDevicesA: TRasEnumDevicesA = nil;
  RasGetCountryInfoA: TRasGetCountryInfoA = nil;
  RasGetEntryPropertiesA: TRasGetEntryPropertiesA = nil;
  RasSetEntryPropertiesA: TRasSetEntryPropertiesA = nil;
  RasRenameEntryA: TRasRenameEntryA = nil;
  RasDeleteEntryA: TRasDeleteEntryA = nil;
  RasValidateEntryNameA: TRasValidateEntryNameA = nil;
{$IFDEF WINNT4PLUS}
  RasGetSubEntryHandleA: TRasGetSubEntryHandleA = nil;
  RasConnectionNotificationA: TRasConnectionNotificationA = nil;
  RasGetSubEntryPropertiesA: TRasGetSubEntryPropertiesA = nil;
  RasSetSubEntryPropertiesA: TRasSetSubEntryPropertiesA = nil;
  RasGetCredentialsA: TRasGetCredentialsA = nil;
  RasSetCredentialsA: TRasSetCredentialsA = nil;
  RasGetAutodialAddressA: TRasGetAutodialAddressA = nil;
  RasSetAutodialAddressA: TRasSetAutodialAddressA = nil;
  RasEnumAutodialAddressesA: TRasEnumAutodialAddressesA = nil;
  RasGetAutodialEnableA: TRasGetAutodialEnableA = nil;
  RasSetAutodialEnableA: TRasSetAutodialEnableA = nil;
  RasGetAutodialParamA: TRasGetAutodialParamA = nil;
  RasSetAutodialParamA: TRasSetAutodialParamA = nil;
{$ENDIF}
{$IFDEF WIN2000PLUS}
  RasGetEapUserDataA: TRasGetEapUserDataA = nil;
  RasSetEapUserDataA: TRasSetEapUserDataA = nil;
  RasGetCustomAuthDataA: TRasGetCustomAuthDataA = nil;
  RasSetCustomAuthDataA: TRasSetCustomAuthDataA = nil;
  RasGetEapUserIdentityA: TRasGetEapUserIdentityA = nil;
  RasFreeEapUserIdentityA: TRasFreeEapUserIdentityA = nil;
{$ENDIF}

//UNICODE versions
  RasDialW: TRasDialW = nil;
  RasEnumConnectionsW: TRasEnumConnectionsW = nil;
  RasEnumEntriesW: TRasEnumEntriesW = nil;
  RasGetConnectStatusW: TRasGetConnectStatusW = nil;
  RasGetErrorStringW: TRasGetErrorStringW = nil;
  RasHangUpW: TRasHangUpW = nil;
  RasGetProjectionInfoW: TRasGetProjectionInfoW = nil;
  RasCreatePhonebookEntryW: TRasCreatePhonebookEntryW = nil;
  RasEditPhonebookEntryW: TRasEditPhonebookEntryW = nil;
  RasSetEntryDialParamsW: TRasSetEntryDialParamsW = nil;
  RasGetEntryDialParamsW: TRasGetEntryDialParamsW = nil;
  RasEnumDevicesW: TRasEnumDevicesW = nil;
  RasGetCountryInfoW: TRasGetCountryInfoW = nil;
  RasGetEntryPropertiesW: TRasGetEntryPropertiesW = nil;
  RasSetEntryPropertiesW: TRasSetEntryPropertiesW = nil;
  RasRenameEntryW: TRasRenameEntryW = nil;
  RasDeleteEntryW: TRasDeleteEntryW = nil;
  RasValidateEntryNameW: TRasValidateEntryNameW = nil;
{$IFDEF WINNT4PLUS}
  RasGetSubEntryHandleW: TRasGetSubEntryHandleW = nil;
  RasConnectionNotificationW: TRasConnectionNotificationW = nil;
  RasGetSubEntryPropertiesW: TRasGetSubEntryPropertiesW = nil;
  RasSetSubEntryPropertiesW: TRasSetSubEntryPropertiesW = nil;
  RasGetCredentialsW: TRasGetCredentialsW = nil;
  RasSetCredentialsW: TRasSetCredentialsW = nil;
  RasGetAutodialAddressW: TRasGetAutodialAddressW = nil;
  RasSetAutodialAddressW: TRasSetAutodialAddressW = nil;
  RasEnumAutodialAddressesW: TRasEnumAutodialAddressesW = nil;
  RasGetAutodialEnableW: TRasGetAutodialEnableW = nil;
  RasSetAutodialEnableW: TRasSetAutodialEnableW = nil;
  RasGetAutodialParamW: TRasGetAutodialParamW = nil;
  RasSetAutodialParamW: TRasSetAutodialParamW = nil;
{$ENDIF}
{$IFDEF WIN2000PLUS}
  RasGetEapUserDataW: TRasGetEapUserDataW = nil;
  RasSetEapUserDataW: TRasSetEapUserDataW = nil;
  RasGetCustomAuthDataW: TRasGetCustomAuthDataW = nil;
  RasSetCustomAuthDataW: TRasSetCustomAuthDataW = nil;
  RasGetEapUserIdentityW: TRasGetEapUserIdentityW = nil;
  RasFreeEapUserIdentityW: TRasFreeEapUserIdentityW = nil;
{$ENDIF}
  libHandle: THandle = 0;
const
  libName = 'RASAPI32.DLL';

procedure InitRAS();
begin
  libHandle := LoadLibrary(libName);
  if libHandle<>0 then begin
//Standard versions default to ANSI
    @RasDial := GetProcAddress(libHandle, 'RasDialA');
    @RasEnumConnections := GetProcAddress(libHandle, 'RasEnumConnectionsA');
    @RasEnumEntries := GetProcAddress(libHandle, 'RasEnumEntriesA');
    @RasGetConnectStatus := GetProcAddress(libHandle, 'RasGetConnectStatusA');
    @RasGetErrorString := GetProcAddress(libHandle, 'RasGetErrorStringA');
    @RasHangUp := GetProcAddress(libHandle, 'RasHangUpA');
    @RasGetProjectionInfo := GetProcAddress(libHandle, 'RasGetProjectionInfoA');
    @RasCreatePhonebookEntry := GetProcAddress(libHandle, 'RasCreatePhonebookEntryA');
    @RasEditPhonebookEntry := GetProcAddress(libHandle, 'RasEditPhonebookEntryA');
    @RasSetEntryDialParams := GetProcAddress(libHandle, 'RasSetEntryDialParamsA');
    @RasGetEntryDialParams := GetProcAddress(libHandle, 'RasGetEntryDialParamsA');
    @RasEnumDevices := GetProcAddress(libHandle, 'RasEnumDevicesA');
    @RasGetCountryInfo := GetProcAddress(libHandle, 'RasGetCountryInfoA');
    @RasGetEntryProperties := GetProcAddress(libHandle, 'RasGetEntryPropertiesA');
    @RasSetEntryProperties := GetProcAddress(libHandle, 'RasSetEntryPropertiesA');
    @RasRenameEntry := GetProcAddress(libHandle, 'RasRenameEntryA');
    @RasDeleteEntry := GetProcAddress(libHandle, 'RasDeleteEntryA');
    @RasValidateEntryName := GetProcAddress(libHandle, 'RasValidateEntryNameA');
{$IFDEF WINNT4PLUS}
    @RasGetSubEntryHandle := GetProcAddress(libHandle, 'RasGetSubEntryHandleA');
    @RasConnectionNotification := GetProcAddress(libHandle, 'RasConnectionNotificationA');
    @RasGetSubEntryProperties := GetProcAddress(libHandle, 'RasGetSubEntryPropertiesA');
    @RasSetSubEntryProperties := GetProcAddress(libHandle, 'RasSetSubEntryPropertiesA');
    @RasGetCredentials := GetProcAddress(libHandle, 'RasGetCredentialsA');
    @RasSetCredentials := GetProcAddress(libHandle, 'RasSetCredentialsA');
    @RasGetAutodialAddress := GetProcAddress(libHandle, 'RasGetAutodialAddressA');
    @RasSetAutodialAddress := GetProcAddress(libHandle, 'RasSetAutodialAddressA');
    @RasEnumAutodialAddresses := GetProcAddress(libHandle, 'RasEnumAutodialAddressesA');
    @RasGetAutodialEnable := GetProcAddress(libHandle, 'RasGetAutodialEnableA');
    @RasSetAutodialEnable := GetProcAddress(libHandle, 'RasSetAutodialEnableA');
    @RasGetAutodialParam := GetProcAddress(libHandle, 'RasGetAutodialParamA');
    @RasSetAutodialParam := GetProcAddress(libHandle, 'RasSetAutodialParamA');
{$ENDIF}
{$IFDEF WIN2000PLUS}
    @RasGetEapUserData := GetProcAddress(libHandle, 'RasGetEapUserDataA');
    @RasSetEapUserData := GetProcAddress(libHandle, 'RasSetEapUserDataA');
    @RasGetCustomAuthData := GetProcAddress(libHandle, 'RasGetCustomAuthDataA');
    @RasSetCustomAuthData := GetProcAddress(libHandle, 'RasSetCustomAuthDataA');
    @RasGetEapUserIdentity := GetProcAddress(libHandle, 'RasGetEapUserIdentityA');
    @RasFreeEapUserIdentity := GetProcAddress(libHandle, 'RasFreeEapUserIdentityA');
{$ENDIF}

//ANSI versions
    @RasDialA := GetProcAddress(libHandle, 'RasDialA');
    @RasEnumConnectionsA := GetProcAddress(libHandle, 'RasEnumConnectionsA');
    @RasEnumEntriesA := GetProcAddress(libHandle, 'RasEnumEntriesA');
    @RasGetConnectStatusA := GetProcAddress(libHandle, 'RasGetConnectStatusA');
    @RasGetErrorStringA := GetProcAddress(libHandle, 'RasGetErrorStringA');
    @RasHangUpA := GetProcAddress(libHandle, 'RasHangUpA');
    @RasGetProjectionInfoA := GetProcAddress(libHandle, 'RasGetProjectionInfoA');
    @RasCreatePhonebookEntryA := GetProcAddress(libHandle, 'RasCreatePhonebookEntryA');
    @RasEditPhonebookEntryA := GetProcAddress(libHandle, 'RasEditPhonebookEntryA');
    @RasSetEntryDialParamsA := GetProcAddress(libHandle, 'RasSetEntryDialParamsA');
    @RasGetEntryDialParamsA := GetProcAddress(libHandle, 'RasGetEntryDialParamsA');
    @RasEnumDevicesA := GetProcAddress(libHandle, 'RasEnumDevicesA');
    @RasGetCountryInfoA := GetProcAddress(libHandle, 'RasGetCountryInfoA');
    @RasGetEntryPropertiesA := GetProcAddress(libHandle, 'RasGetEntryPropertiesA');
    @RasSetEntryPropertiesA := GetProcAddress(libHandle, 'RasSetEntryPropertiesA');
    @RasRenameEntryA := GetProcAddress(libHandle, 'RasRenameEntryA');
    @RasDeleteEntryA := GetProcAddress(libHandle, 'RasDeleteEntryA');
    @RasValidateEntryNameA := GetProcAddress(libHandle, 'RasValidateEntryNameA');
{$IFDEF WINNT4PLUS}
    @RasGetSubEntryHandleA := GetProcAddress(libHandle, 'RasGetSubEntryHandleA');
    @RasConnectionNotificationA := GetProcAddress(libHandle, 'RasConnectionNotificationA');
    @RasGetSubEntryPropertiesA := GetProcAddress(libHandle, 'RasGetSubEntryPropertiesA');
    @RasSetSubEntryPropertiesA := GetProcAddress(libHandle, 'RasSetSubEntryPropertiesA');
    @RasGetCredentialsA := GetProcAddress(libHandle, 'RasGetCredentialsA');
    @RasSetCredentialsA := GetProcAddress(libHandle, 'RasSetCredentialsA');
    @RasGetAutodialAddressA := GetProcAddress(libHandle, 'RasGetAutodialAddressA');
    @RasSetAutodialAddressA := GetProcAddress(libHandle, 'RasSetAutodialAddressA');
    @RasEnumAutodialAddressesA := GetProcAddress(libHandle, 'RasEnumAutodialAddressesA');
    @RasGetAutodialEnableA := GetProcAddress(libHandle, 'RasGetAutodialEnableA');
    @RasSetAutodialEnableA := GetProcAddress(libHandle, 'RasSetAutodialEnableA');
    @RasGetAutodialParamA := GetProcAddress(libHandle, 'RasGetAutodialParamA');
    @RasSetAutodialParamA := GetProcAddress(libHandle, 'RasSetAutodialParamA');
{$ENDIF}
{$IFDEF WIN2000PLUS}
    @RasGetEapUserDataA := GetProcAddress(libHandle, 'RasGetEapUserDataA');
    @RasSetEapUserDataA := GetProcAddress(libHandle, 'RasSetEapUserDataA');
    @RasGetCustomAuthDataA := GetProcAddress(libHandle, 'RasGetCustomAuthDataA');
    @RasSetCustomAuthDataA := GetProcAddress(libHandle, 'RasSetCustomAuthDataA');
    @RasGetEapUserIdentityA := GetProcAddress(libHandle, 'RasGetEapUserIdentityA');
    @RasFreeEapUserIdentityA := GetProcAddress(libHandle, 'RasFreeEapUserIdentityA');
{$ENDIF}

//UNICODE versions
    @RasDialW := GetProcAddress(libHandle, 'RasDialW');
    @RasEnumConnectionsW := GetProcAddress(libHandle, 'RasEnumConnectionsW');
    @RasEnumEntriesW := GetProcAddress(libHandle, 'RasEnumEntriesW');
    @RasGetConnectStatusW := GetProcAddress(libHandle, 'RasGetConnectStatusW');
    @RasGetErrorStringW := GetProcAddress(libHandle, 'RasGetErrorStringW');
    @RasHangUpW := GetProcAddress(libHandle, 'RasHangUpW');
    @RasGetProjectionInfoW := GetProcAddress(libHandle, 'RasGetProjectionInfoW');
    @RasCreatePhonebookEntryW := GetProcAddress(libHandle, 'RasCreatePhonebookEntryW');
    @RasEditPhonebookEntryW := GetProcAddress(libHandle, 'RasEditPhonebookEntryW');
    @RasSetEntryDialParamsW := GetProcAddress(libHandle, 'RasSetEntryDialParamsW');
    @RasGetEntryDialParamsW := GetProcAddress(libHandle, 'RasGetEntryDialParamsW');
    @RasEnumDevicesW := GetProcAddress(libHandle, 'RasEnumDevicesW');
    @RasGetCountryInfoW := GetProcAddress(libHandle, 'RasGetCountryInfoW');
    @RasGetEntryPropertiesW := GetProcAddress(libHandle, 'RasGetEntryPropertiesW');
    @RasSetEntryPropertiesW := GetProcAddress(libHandle, 'RasSetEntryPropertiesW');
    @RasRenameEntryW := GetProcAddress(libHandle, 'RasRenameEntryW');
    @RasDeleteEntryW := GetProcAddress(libHandle, 'RasDeleteEntryW');
    @RasValidateEntryNameW := GetProcAddress(libHandle, 'RasValidateEntryNameW');
{$IFDEF WINNT4PLUS}
    @RasGetSubEntryHandleW := GetProcAddress(libHandle, 'RasGetSubEntryHandleW');
    @RasConnectionNotificationW := GetProcAddress(libHandle, 'RasConnectionNotificationW');
    @RasGetSubEntryPropertiesW := GetProcAddress(libHandle, 'RasGetSubEntryPropertiesW');
    @RasSetSubEntryPropertiesW := GetProcAddress(libHandle, 'RasSetSubEntryPropertiesW');
    @RasGetCredentialsW := GetProcAddress(libHandle, 'RasGetCredentialsW');
    @RasSetCredentialsW := GetProcAddress(libHandle, 'RasSetCredentialsW');
    @RasGetAutodialAddressW := GetProcAddress(libHandle, 'RasGetAutodialAddressW');
    @RasSetAutodialAddressW := GetProcAddress(libHandle, 'RasSetAutodialAddressW');
    @RasEnumAutodialAddressesW := GetProcAddress(libHandle, 'RasEnumAutodialAddressesW');
    @RasGetAutodialEnableW := GetProcAddress(libHandle, 'RasGetAutodialEnableW');
    @RasSetAutodialEnableW := GetProcAddress(libHandle, 'RasSetAutodialEnableW');
    @RasGetAutodialParamW := GetProcAddress(libHandle, 'RasGetAutodialParamW');
    @RasSetAutodialParamW := GetProcAddress(libHandle, 'RasSetAutodialParamW');
{$ENDIF}
{$IFDEF WIN2000PLUS}
    @RasGetEapUserDataW := GetProcAddress(libHandle, 'RasGetEapUserDataW');
    @RasSetEapUserDataW := GetProcAddress(libHandle, 'RasSetEapUserDataW');
    @RasGetCustomAuthDataW := GetProcAddress(libHandle, 'RasGetCustomAuthDataW');
    @RasSetCustomAuthDataW := GetProcAddress(libHandle, 'RasSetCustomAuthDataW');
    @RasGetEapUserIdentityW := GetProcAddress(libHandle, 'RasGetEapUserIdentityW');
    @RasFreeEapUserIdentityW := GetProcAddress(libHandle, 'RasFreeEapUserIdentityW');
{$ENDIF}
  end;
end;

